/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.ui.prototype;

import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createAccelerationXSensor;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createAccelerationYSensor;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createAccelerationZSensor;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createActuatorDigits;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createAngularVelocityXSensor;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createAngularVelocityYSensor;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createAngularVelocityZSensor;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createButton1;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createButton2;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createButton3;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createButton4;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createButtonL1;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createButtonL2;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createButtonR1;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createButtonR2;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createCameraConnected;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createCameraDetectionStateLeft;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createCameraDetectionStateRight;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createCameraDistanceLeft;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createCameraDistanceRight;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createCameraLastUpdate;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createCameraYawAngle;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createControlCenterDownstreamInput;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createControlCenterUpstreamOutput;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createLEDButtonBActuator;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createLEDButtonGActuator;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createLEDButtonRActuator;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createLaserRangeSensor;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createLeftStickX;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createLeftStickY;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createMotorControlInput;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createMotorControlOutput;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRightStickX;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRightStickY;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonA;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonB;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonHome;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonL1;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonL3;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonR1;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonR3;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonSelect;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonStart;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonX;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleButtonY;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleDPadDown;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleDPadLeft;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleDPadRight;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleDPadUp;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleL2Position;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleLeftStickX;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleLeftStickY;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleMagnitudeStrong;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleMagnitudeWeak;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleR2Position;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleRightStickX;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleRightStickY;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRumbleSimpleRumbleFeaturePosition;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createUltraSonicSensor;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_HEIGHT;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_WIDTH;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUIUtils.setConnectorPosition;
import static org.fortiss.tooling.base.utils.LayoutDataUtils.setNodeBounds;
import static org.fortiss.tooling.base.utils.LayoutDataUtils.setNodePosition;
import static org.fortiss.tooling.base.utils.LayoutModelElementFactory.createRectangle;

import org.fortiss.af3.platform.raspberry.model.ActuatorPWM;
import org.fortiss.af3.platform.raspberry.model.CanBus;
import org.fortiss.af3.platform.raspberry.model.CanConnector;
import org.fortiss.af3.platform.raspberry.model.ConsoleOutput;
import org.fortiss.af3.platform.raspberry.model.RaspberryPi;
import org.fortiss.af3.platform.raspberry.model.brick.AccelerationXSensor;
import org.fortiss.af3.platform.raspberry.model.brick.AccelerationYSensor;
import org.fortiss.af3.platform.raspberry.model.brick.AccelerationZSensor;
import org.fortiss.af3.platform.raspberry.model.brick.ActuatorDigits;
import org.fortiss.af3.platform.raspberry.model.brick.AngularVelocityXSensor;
import org.fortiss.af3.platform.raspberry.model.brick.AngularVelocityYSensor;
import org.fortiss.af3.platform.raspberry.model.brick.AngularVelocityZSensor;
import org.fortiss.af3.platform.raspberry.model.brick.LEDButtonBActuator;
import org.fortiss.af3.platform.raspberry.model.brick.LEDButtonGActuator;
import org.fortiss.af3.platform.raspberry.model.brick.LEDButtonRActuator;
import org.fortiss.af3.platform.raspberry.model.brick.LaserRangeSensor;
import org.fortiss.af3.platform.raspberry.model.brick.UltraSonicSensor;
import org.fortiss.af3.platform.raspberry.model.camera.CameraConnected;
import org.fortiss.af3.platform.raspberry.model.camera.CameraDetectionStateLeft;
import org.fortiss.af3.platform.raspberry.model.camera.CameraDetectionStateRight;
import org.fortiss.af3.platform.raspberry.model.camera.CameraDistanceLeft;
import org.fortiss.af3.platform.raspberry.model.camera.CameraDistanceRight;
import org.fortiss.af3.platform.raspberry.model.camera.CameraLastUpdate;
import org.fortiss.af3.platform.raspberry.model.camera.CameraYawAngle;
import org.fortiss.af3.platform.raspberry.model.controlcenter.ControlCenterDownstreamInput;
import org.fortiss.af3.platform.raspberry.model.controlcenter.ControlCenterUpstreamOutput;
import org.fortiss.af3.platform.raspberry.model.gamepad.Button1;
import org.fortiss.af3.platform.raspberry.model.gamepad.Button2;
import org.fortiss.af3.platform.raspberry.model.gamepad.Button3;
import org.fortiss.af3.platform.raspberry.model.gamepad.Button4;
import org.fortiss.af3.platform.raspberry.model.gamepad.ButtonL1;
import org.fortiss.af3.platform.raspberry.model.gamepad.ButtonL2;
import org.fortiss.af3.platform.raspberry.model.gamepad.ButtonR1;
import org.fortiss.af3.platform.raspberry.model.gamepad.ButtonR2;
import org.fortiss.af3.platform.raspberry.model.gamepad.Left_StickX_Position;
import org.fortiss.af3.platform.raspberry.model.gamepad.Left_StickY_Position;
import org.fortiss.af3.platform.raspberry.model.gamepad.Right_StickX_Position;
import org.fortiss.af3.platform.raspberry.model.gamepad.Right_StickY_Position;
import org.fortiss.af3.platform.raspberry.model.motorcontrol.MotorControlInput;
import org.fortiss.af3.platform.raspberry.model.motorcontrol.MotorControlOutput;
import org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonA;
import org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonB;
import org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonHome;
import org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonL3;
import org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonR3;
import org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonSelect;
import org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonStart;
import org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonX;
import org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonY;
import org.fortiss.af3.platform.raspberry.model.rumblepad.DPadDown;
import org.fortiss.af3.platform.raspberry.model.rumblepad.DPadLeft;
import org.fortiss.af3.platform.raspberry.model.rumblepad.DPadRight;
import org.fortiss.af3.platform.raspberry.model.rumblepad.DPadUp;
import org.fortiss.af3.platform.raspberry.model.rumblepad.L2_Position;
import org.fortiss.af3.platform.raspberry.model.rumblepad.R2_Position;
import org.fortiss.af3.platform.raspberry.model.rumblepad.RumbleMagnitudeStrong;
import org.fortiss.af3.platform.raspberry.model.rumblepad.RumbleMagnitudeWeak;
import org.fortiss.af3.platform.raspberry.model.rumblepad.SimpleRumbleFeature;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VFollowerInput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VFollowerOutput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VFusePlatoonInput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VFusePlatoonOutput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VLeaderHeartbeatInput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VLeaderHeartbeatOutput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VLeavePlatoonInput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VLeavePlatoonOutput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VNewLeaderInput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VNewLeaderOutput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VPlatoonInput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VPlatoonOutput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VSplitInput;
import org.fortiss.af3.platform.raspberry.model.v2v.V2VSplitOutput;
import org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory;
import org.fortiss.tooling.kernel.extension.base.PrototypeProviderBase;
import org.fortiss.tooling.kernel.extension.data.PrototypeCategory;

/** {@link PrototypeProviderBase} for {@link RaspberryPi} model elements. */
public class ProtoypeProvider extends PrototypeProviderBase {
	/** Stores the category to display for these prototypes. */
	protected final static String CATEGORY_NAME = "RaspberryPI Platform";
	/** Category for gamepad elements. */
	protected final static String GAMEPAD = "Gamepad";
	/** Category for rumblepad elements. */
	protected final static String RUMBLEPAD = "Rumblepad";
	/** Category for brick elements. */
	protected final static String BRICK = "Bricklets";
	/** Category for motor control elements. */
	protected final static String MOTOR_CONTROL = "Motor Control";
	/** Category for camera server elements. */
	protected final static String CAMERA_SERVER = "Camera Server";
	/** Category for control center elements. */
	protected final static String CONTROL_CENTER = "Control Center";
	/** Category for control center elements. */
	protected final static String V2V_COM = "V2V Communication";

	/** {@inheritDoc} */
	@Override
	protected void registerPrototypes() {
		RaspberryPi pi = RaspberryModelElementFactory.createRaspberryPi();
		registerPrimaryPrototype("RaspberryPi", pi, CATEGORY_NAME);
		PrototypeCategory prototypeCategory = registerPrototypeCategory(CATEGORY_NAME);
		registerPrototypeCategory(prototypeCategory, GAMEPAD);
		registerPrototypeCategory(prototypeCategory, RUMBLEPAD);
		registerPrototypeCategory(prototypeCategory, BRICK);
		registerPrototypeCategory(prototypeCategory, MOTOR_CONTROL);
		registerPrototypeCategory(prototypeCategory, CAMERA_SERVER);
		registerPrototypeCategory(prototypeCategory, CONTROL_CENTER);
		registerPrototypeCategory(prototypeCategory, V2V_COM);

		CanBus bus = RaspberryModelElementFactory.createCanBus();
		setNodeBounds(bus,
				createRectangle(0, 0, DEFAULT_SHAPE_MINIMUM_WIDTH, DEFAULT_SHAPE_MINIMUM_HEIGHT));
		registerPrototype("CanBus", bus, CATEGORY_NAME);

		CanConnector con = RaspberryModelElementFactory.createCanConnector();
		setNodePosition(con, 0, 0);
		setConnectorPosition(con, 0, 0);
		registerPrototype("CanConnector", con, CATEGORY_NAME);

		ActuatorPWM pwm = RaspberryModelElementFactory.createActuatorPWM();
		setNodePosition(pwm, 0, 0);
		setConnectorPosition(pwm, 0, 0);
		registerPrototype("PWM_Actuator", pwm, CATEGORY_NAME);

		ConsoleOutput printf = RaspberryModelElementFactory.createConsoleOutput();
		setNodePosition(printf, 0, 0);
		setConnectorPosition(printf, 0, 0);
		registerPrototype("Console_Output", printf, CATEGORY_NAME);

		registerBrickElements();
		registerRumblepadElements();
		registerMotorControlElements();
		registerCameraElements();
		registerGamepadElements();
		registerControlCenterElements();
		registerV2VComElements();
	}

	/**
	 * 
	 */
	private void registerV2VComElements() {
		V2VPlatoonInput input = RaspberryModelElementFactory.createV2vPlatoonInput();
		setNodePosition(input, 0, 0);
		setConnectorPosition(input, 0, 0);
		registerPrototype("V2V Platoon Input", input, V2V_COM);

		V2VPlatoonOutput output = RaspberryModelElementFactory.createV2vPlatoonOutput();
		setNodePosition(output, 0, 0);
		setConnectorPosition(output, 0, 0);
		registerPrototype("V2V Platoon Output", output, V2V_COM);

		V2VLeaderHeartbeatInput input1 =
				RaspberryModelElementFactory.createV2vLeaderHeartbeatInput();
		setNodePosition(input1, 0, 0);
		setConnectorPosition(input1, 0, 0);
		registerPrototype("V2V Leader Heartbeat Input", input1, V2V_COM);

		V2VLeaderHeartbeatOutput output1 =
				RaspberryModelElementFactory.createV2vLeaderHeartbeatOutput();
		setNodePosition(output1, 0, 0);
		setConnectorPosition(output1, 0, 0);
		registerPrototype("V2V Leader Heartbeat Output", output1, V2V_COM);

		V2VFollowerInput input2 = RaspberryModelElementFactory.createV2vFollowerInput();
		setNodePosition(input2, 0, 0);
		setConnectorPosition(input2, 0, 0);
		registerPrototype("V2V Follower Input", input2, V2V_COM);

		V2VFollowerOutput output2 = RaspberryModelElementFactory.createV2vFollowerOutput();
		setNodePosition(output2, 0, 0);
		setConnectorPosition(output2, 0, 0);
		registerPrototype("V2V Follower Output", output2, V2V_COM);

		V2VLeavePlatoonInput input3 = RaspberryModelElementFactory.createV2vLeavePlatoonInput();
		setNodePosition(input3, 0, 0);
		setConnectorPosition(input3, 0, 0);
		registerPrototype("V2V Leave Platoon Input", input3, V2V_COM);

		V2VLeavePlatoonOutput output3 = RaspberryModelElementFactory.createV2vLeavePlatoonOutput();
		setNodePosition(output3, 0, 0);
		setConnectorPosition(output3, 0, 0);
		registerPrototype("V2V Leave Platoon Output", output3, V2V_COM);

		V2VSplitInput input4 = RaspberryModelElementFactory.createV2vSplitInput();
		setNodePosition(input4, 0, 0);
		setConnectorPosition(input4, 0, 0);
		registerPrototype("V2V Split Input", input4, V2V_COM);

		V2VSplitOutput output4 = RaspberryModelElementFactory.createV2vSplitOutput();
		setNodePosition(output4, 0, 0);
		setConnectorPosition(output4, 0, 0);
		registerPrototype("V2V Split Output", output4, V2V_COM);

		V2VNewLeaderInput input5 = RaspberryModelElementFactory.createV2vNewLeaderInput();
		setNodePosition(input5, 0, 0);
		setConnectorPosition(input5, 0, 0);
		registerPrototype("V2V New Leader Input", input5, V2V_COM);

		V2VNewLeaderOutput output5 = RaspberryModelElementFactory.createV2vNewLeaderOutput();
		setNodePosition(output5, 0, 0);
		setConnectorPosition(output5, 0, 0);
		registerPrototype("V2V New Leader Output", output5, V2V_COM);

		V2VFusePlatoonInput input6 = RaspberryModelElementFactory.createV2vFusePlatoonInput();
		setNodePosition(input6, 0, 0);
		setConnectorPosition(input6, 0, 0);
		registerPrototype("V2V Fuse Platoon Input", input6, V2V_COM);

		V2VFusePlatoonOutput output6 = RaspberryModelElementFactory.createV2vFusePlatoonOutput();
		setNodePosition(output6, 0, 0);
		setConnectorPosition(output6, 0, 0);
		registerPrototype("V2V Fuse Platoon Output", output6, V2V_COM);
	}

	/** Register elements for control center. */
	private void registerControlCenterElements() {
		ControlCenterDownstreamInput input = createControlCenterDownstreamInput();
		setNodePosition(input, 0, 0);
		setConnectorPosition(input, 0, 0);
		registerPrototype("CC Input", input, CONTROL_CENTER);

		ControlCenterUpstreamOutput output = createControlCenterUpstreamOutput();
		setNodePosition(output, 0, 0);
		setConnectorPosition(output, 0, 0);
		registerPrototype("CC Output", output, CONTROL_CENTER);
	}

	/** Register brick actuators and sensors. */
	private void registerBrickElements() {
		ActuatorDigits digits = createActuatorDigits();
		setNodePosition(digits, 0, 0);
		setConnectorPosition(digits, 0, 0);
		registerPrototype("Segment_Digits", digits, BRICK);

		LEDButtonRActuator ledRButton = createLEDButtonRActuator();
		setNodePosition(ledRButton, 0, 0);
		setConnectorPosition(ledRButton, 0, 0);
		registerPrototype("LED_Button_red", ledRButton, BRICK);
		LEDButtonGActuator ledGButton = createLEDButtonGActuator();
		setNodePosition(ledGButton, 0, 0);
		setConnectorPosition(ledGButton, 0, 0);
		registerPrototype("LED_Button_green", ledGButton, BRICK);
		LEDButtonBActuator ledBButton = createLEDButtonBActuator();
		setNodePosition(ledBButton, 0, 0);
		setConnectorPosition(ledBButton, 0, 0);
		registerPrototype("LED_Button_blue", ledBButton, BRICK);

		UltraSonicSensor uss = createUltraSonicSensor();
		setNodePosition(uss, 0, 0);
		setConnectorPosition(uss, 0, 0);
		registerPrototype(uss.getName(), uss, BRICK);

		LaserRangeSensor lsr = createLaserRangeSensor();
		setNodePosition(lsr, 0, 0);
		setConnectorPosition(lsr, 0, 0);
		registerPrototype(lsr.getName(), lsr, BRICK);

		AccelerationXSensor accX = createAccelerationXSensor();
		setNodePosition(accX, 0, 0);
		setConnectorPosition(accX, 0, 0);
		registerPrototype(accX.getName(), accX, BRICK);

		AccelerationYSensor accY = createAccelerationYSensor();
		setNodePosition(accY, 0, 0);
		setConnectorPosition(accY, 0, 0);
		registerPrototype(accY.getName(), accY, BRICK);

		AccelerationZSensor accZ = createAccelerationZSensor();
		setNodePosition(accZ, 0, 0);
		setConnectorPosition(accZ, 0, 0);
		registerPrototype(accZ.getName(), accZ, BRICK);

		AngularVelocityXSensor avX = createAngularVelocityXSensor();
		setNodePosition(avX, 0, 0);
		setConnectorPosition(avX, 0, 0);
		registerPrototype(avX.getName(), avX, BRICK);

		AngularVelocityYSensor avY = createAngularVelocityYSensor();
		setNodePosition(avY, 0, 0);
		setConnectorPosition(avY, 0, 0);
		registerPrototype(avY.getName(), avY, BRICK);

		AngularVelocityZSensor avZ = createAngularVelocityZSensor();
		setNodePosition(avZ, 0, 0);
		setConnectorPosition(avZ, 0, 0);
		registerPrototype(avZ.getName(), avZ, BRICK);
	}

	/** Register rumblepad elements. */
	private void registerRumblepadElements() {
		org.fortiss.af3.platform.raspberry.model.rumblepad.Left_StickX_Position rplsx =
				createRumbleLeftStickX();
		setNodePosition(rplsx, 0, 0);
		setConnectorPosition(rplsx, 0, 0);
		registerPrototype(rplsx.getName(), rplsx, RUMBLEPAD);

		registerRumblepadSticks();
		registerRumblepadButtons();
	}

	/** Register rumblepad buttons. */
	private void registerRumblepadButtons() {
		org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonL1 rpl1 = createRumbleButtonL1();
		setNodePosition(rpl1, 0, 0);
		setConnectorPosition(rpl1, 0, 0);
		registerPrototype(rpl1.getName(), rpl1, RUMBLEPAD);

		org.fortiss.af3.platform.raspberry.model.rumblepad.ButtonR1 rpr1 = createRumbleButtonR1();
		setNodePosition(rpr1, 0, 0);
		setConnectorPosition(rpr1, 0, 0);
		registerPrototype(rpr1.getName(), rpr1, RUMBLEPAD);

		L2_Position rpl2 = createRumbleL2Position();
		setNodePosition(rpl2, 0, 0);
		setConnectorPosition(rpl2, 0, 0);
		registerPrototype(rpl2.getName(), rpl2, RUMBLEPAD);

		R2_Position rpr2 = createRumbleR2Position();
		setNodePosition(rpr2, 0, 0);
		setConnectorPosition(rpr2, 0, 0);
		registerPrototype(rpr2.getName(), rpr2, RUMBLEPAD);

		ButtonA rpba = createRumbleButtonA();
		setNodePosition(rpba, 0, 0);
		setConnectorPosition(rpba, 0, 0);
		registerPrototype(rpba.getName(), rpba, RUMBLEPAD);

		ButtonB rpbb = createRumbleButtonB();
		setNodePosition(rpbb, 0, 0);
		setConnectorPosition(rpbb, 0, 0);
		registerPrototype(rpbb.getName(), rpbb, RUMBLEPAD);

		ButtonX rpbx = createRumbleButtonX();
		setNodePosition(rpbx, 0, 0);
		setConnectorPosition(rpbx, 0, 0);
		registerPrototype(rpbx.getName(), rpbx, RUMBLEPAD);

		ButtonY rpby = createRumbleButtonY();
		setNodePosition(rpby, 0, 0);
		setConnectorPosition(rpby, 0, 0);
		registerPrototype(rpby.getName(), rpby, RUMBLEPAD);

		ButtonStart rpbst = createRumbleButtonStart();
		setNodePosition(rpbst, 0, 0);
		setConnectorPosition(rpbst, 0, 0);
		registerPrototype(rpbst.getName(), rpbst, RUMBLEPAD);

		ButtonSelect rpbse = createRumbleButtonSelect();
		setNodePosition(rpbse, 0, 0);
		setConnectorPosition(rpbse, 0, 0);
		registerPrototype(rpbse.getName(), rpbse, RUMBLEPAD);

		ButtonHome rpbh = createRumbleButtonHome();
		setNodePosition(rpbh, 0, 0);
		setConnectorPosition(rpbh, 0, 0);
		registerPrototype(rpbh.getName(), rpbh, RUMBLEPAD);

		SimpleRumbleFeature srf = createRumbleSimpleRumbleFeaturePosition();
		setNodePosition(srf, 0, 0);
		setConnectorPosition(srf, 0, 0);
		registerPrototype(srf.getName(), srf, RUMBLEPAD);

		RumbleMagnitudeStrong rms = createRumbleMagnitudeStrong();
		setNodePosition(rms, 0, 0);
		setConnectorPosition(rms, 0, 0);
		registerPrototype(rms.getName(), rms, RUMBLEPAD);

		RumbleMagnitudeWeak rmw = createRumbleMagnitudeWeak();
		setNodePosition(rmw, 0, 0);
		setConnectorPosition(rmw, 0, 0);
		registerPrototype(rmw.getName(), rmw, RUMBLEPAD);
	}

	/** Register rumblepad sticks and D-pad. */
	private void registerRumblepadSticks() {
		org.fortiss.af3.platform.raspberry.model.rumblepad.Left_StickY_Position rplsy =
				createRumbleLeftStickY();
		setNodePosition(rplsy, 0, 0);
		setConnectorPosition(rplsy, 0, 0);
		registerPrototype(rplsy.getName(), rplsy, RUMBLEPAD);

		org.fortiss.af3.platform.raspberry.model.rumblepad.Right_StickX_Position rprsx =
				createRumbleRightStickX();
		setNodePosition(rprsx, 0, 0);
		setConnectorPosition(rprsx, 0, 0);
		registerPrototype(rprsx.getName(), rprsx, RUMBLEPAD);

		org.fortiss.af3.platform.raspberry.model.rumblepad.Right_StickY_Position rprsy =
				createRumbleRightStickY();
		setNodePosition(rprsy, 0, 0);
		setConnectorPosition(rprsy, 0, 0);
		registerPrototype(rprsy.getName(), rprsy, RUMBLEPAD);

		DPadUp rpdpu = createRumbleDPadUp();
		setNodePosition(rpdpu, 0, 0);
		setConnectorPosition(rpdpu, 0, 0);
		registerPrototype(rpdpu.getName(), rpdpu, RUMBLEPAD);

		DPadDown rpdpd = createRumbleDPadDown();
		setNodePosition(rpdpd, 0, 0);
		setConnectorPosition(rpdpd, 0, 0);
		registerPrototype(rpdpd.getName(), rpdpd, RUMBLEPAD);

		DPadLeft rpdpl = createRumbleDPadLeft();
		setNodePosition(rpdpl, 0, 0);
		setConnectorPosition(rpdpl, 0, 0);
		registerPrototype(rpdpl.getName(), rpdpl, RUMBLEPAD);

		DPadRight rpdpr = createRumbleDPadRight();
		setNodePosition(rpdpr, 0, 0);
		setConnectorPosition(rpdpr, 0, 0);
		registerPrototype(rpdpr.getName(), rpdpr, RUMBLEPAD);

		ButtonL3 rpl3 = createRumbleButtonL3();
		setNodePosition(rpl3, 0, 0);
		setConnectorPosition(rpl3, 0, 0);
		registerPrototype(rpl3.getName(), rpl3, RUMBLEPAD);

		ButtonR3 rpr3 = createRumbleButtonR3();
		setNodePosition(rpr3, 0, 0);
		setConnectorPosition(rpr3, 0, 0);
		registerPrototype(rpr3.getName(), rpr3, RUMBLEPAD);
	}

	/** Register motor controller elements. */
	private void registerMotorControlElements() {
		MotorControlInput mci = createMotorControlInput();
		setNodePosition(mci, 0, 0);
		setConnectorPosition(mci, 0, 0);
		registerPrototype(mci.getName(), mci, MOTOR_CONTROL);

		MotorControlOutput mco = createMotorControlOutput();
		setNodePosition(mco, 0, 0);
		setConnectorPosition(mco, 0, 0);
		registerPrototype(mco.getName(), mco, MOTOR_CONTROL);
	}

	/** Register camera elements. */
	private void registerCameraElements() {
		CameraConnected cc = createCameraConnected();
		setNodePosition(cc, 0, 0);
		setConnectorPosition(cc, 0, 0);
		registerPrototype(cc.getName(), cc, CAMERA_SERVER);

		CameraLastUpdate clu = createCameraLastUpdate();
		setNodePosition(clu, 0, 0);
		setConnectorPosition(clu, 0, 0);
		registerPrototype(clu.getName(), clu, CAMERA_SERVER);

		CameraDistanceLeft cdl = createCameraDistanceLeft();
		setNodePosition(cdl, 0, 0);
		setConnectorPosition(cdl, 0, 0);
		registerPrototype(cdl.getName(), cdl, CAMERA_SERVER);

		CameraDistanceRight cdr = createCameraDistanceRight();
		setNodePosition(cdr, 0, 0);
		setConnectorPosition(cdr, 0, 0);
		registerPrototype(cdr.getName(), cdr, CAMERA_SERVER);

		CameraDetectionStateLeft cdsl = createCameraDetectionStateLeft();
		setNodePosition(cdsl, 0, 0);
		setConnectorPosition(cdsl, 0, 0);
		registerPrototype(cdsl.getName(), cdsl, CAMERA_SERVER);

		CameraDetectionStateRight cdsr = createCameraDetectionStateRight();
		setNodePosition(cdsr, 0, 0);
		setConnectorPosition(cdsr, 0, 0);
		registerPrototype(cdsr.getName(), cdsr, CAMERA_SERVER);

		CameraYawAngle cya = createCameraYawAngle();
		setNodePosition(cya, 0, 0);
		setConnectorPosition(cya, 0, 0);
		registerPrototype(cya.getName(), cya, CAMERA_SERVER);
	}

	/** Register gamepad elements. */
	private void registerGamepadElements() {
		Button1 button1 = createButton1();
		setNodePosition(button1, 0, 0);
		setConnectorPosition(button1, 0, 0);
		registerPrototype(button1.getName(), button1, GAMEPAD);

		Button2 button2 = createButton2();
		setNodePosition(button2, 0, 0);
		setConnectorPosition(button2, 0, 0);
		registerPrototype(button2.getName(), button2, GAMEPAD);

		Button3 button3 = createButton3();
		setNodePosition(button3, 0, 0);
		setConnectorPosition(button3, 0, 0);
		registerPrototype(button3.getName(), button3, GAMEPAD);

		Button4 button4 = createButton4();
		setNodePosition(button4, 0, 0);
		setConnectorPosition(button4, 0, 0);
		registerPrototype(button4.getName(), button4, GAMEPAD);

		ButtonL1 buttonL1 = createButtonL1();
		setNodePosition(buttonL1, 0, 0);
		setConnectorPosition(buttonL1, 0, 0);
		registerPrototype(buttonL1.getName(), buttonL1, GAMEPAD);

		ButtonL2 buttonL2 = createButtonL2();
		setNodePosition(buttonL2, 0, 0);
		setConnectorPosition(buttonL2, 0, 0);
		registerPrototype(buttonL2.getName(), buttonL2, GAMEPAD);

		ButtonR1 buttonR1 = createButtonR1();
		setNodePosition(buttonR1, 0, 0);
		setConnectorPosition(buttonR1, 0, 0);
		registerPrototype(buttonR1.getName(), buttonR1, GAMEPAD);

		ButtonR2 buttonR2 = createButtonR2();
		setNodePosition(buttonR2, 0, 0);
		setConnectorPosition(buttonR2, 0, 0);
		registerPrototype(buttonR2.getName(), buttonR2, GAMEPAD);

		Left_StickX_Position Left_StickX = createLeftStickX();
		setNodePosition(Left_StickX, 0, 0);
		setConnectorPosition(Left_StickX, 0, 0);
		registerPrototype(Left_StickX.getName(), Left_StickX, GAMEPAD);

		Left_StickY_Position Left_StickY = createLeftStickY();
		setNodePosition(Left_StickY, 0, 0);
		setConnectorPosition(Left_StickY, 0, 0);
		registerPrototype(Left_StickY.getName(), Left_StickY, GAMEPAD);

		Right_StickX_Position Right_StickX = createRightStickX();
		setNodePosition(Right_StickX, 0, 0);
		setConnectorPosition(Right_StickX, 0, 0);
		registerPrototype(Right_StickX.getName(), Right_StickX, GAMEPAD);

		Right_StickY_Position Right_StickY = createRightStickY();
		setNodePosition(Right_StickY, 0, 0);
		setConnectorPosition(Right_StickY, 0, 0);
		registerPrototype(Right_StickY.getName(), Right_StickY, GAMEPAD);
	}
}
